{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Account activated" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-message page-message-success page-activation-done">
  <div class="container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">check</span>
      </div>

      <div class="message-body">
        <p class="lead">{{ message }}</p>
        <p>{% trans "You can now sign in to finish setting up your account and to participate in or start new discussions." %}</p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}