{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "You are banned" %} | {{ block.super }}{% endblock %}


{% block meta-description %}
{% trans "You are banned from accessing this page." %}
{% endblock meta-description %}


{% block content %}
<div class="page page-error page-error-403 page-error-banned">
  <div class="container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">highlight_off</span>
      </div>

      <div class="message-body">
        {% if ban.message.html %}
        <div class="lead">
          {{ ban.message.html|safe }}
        </div>
        {% else %}
        <p class="lead">
          {{ ban.message.plain }}
        </p>
        {% endif %}

        <p className="message-footnote">
          {% if ban.expires_on %}
            {% blocktrans trimmed with expires_on=ban.expires_on|date:"DATETIME_FORMAT" %}
              This ban expires on {{ expires_on }}.
            {% endblocktrans %}
          {% else %}
            {% trans "This ban is permanent." %}
          {% endif %}
        </p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}
